// Copyright 2016-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//     http://aws.amazon.com/apache2.0/
//
// or in the "license" file accompanying this file. This file is distributed
// on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
// express or implied. See the License for the specific language governing
// permissions and limitations under the License.

package operations

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"net/http"
	"time"

	"golang.org/x/net/context"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/runtime"
	cr "github.com/go-openapi/runtime/client"

	strfmt "github.com/go-openapi/strfmt"
)

// NewStreamInstancesParams creates a new StreamInstancesParams object
// with the default values initialized.
func NewStreamInstancesParams() *StreamInstancesParams {
	var ()
	return &StreamInstancesParams{

		timeout: cr.DefaultTimeout,
	}
}

// NewStreamInstancesParamsWithTimeout creates a new StreamInstancesParams object
// with the default values initialized, and the ability to set a timeout on a request
func NewStreamInstancesParamsWithTimeout(timeout time.Duration) *StreamInstancesParams {
	var ()
	return &StreamInstancesParams{

		timeout: timeout,
	}
}

// NewStreamInstancesParamsWithContext creates a new StreamInstancesParams object
// with the default values initialized, and the ability to set a context for a request
func NewStreamInstancesParamsWithContext(ctx context.Context) *StreamInstancesParams {
	var ()
	return &StreamInstancesParams{

		Context: ctx,
	}
}

// NewStreamInstancesParamsWithHTTPClient creates a new StreamInstancesParams object
// with the default values initialized, and the ability to set a custom HTTPClient for a request
func NewStreamInstancesParamsWithHTTPClient(client *http.Client) *StreamInstancesParams {
	var ()
	return &StreamInstancesParams{
		HTTPClient: client,
	}
}

/*StreamInstancesParams contains all the parameters to send to the API endpoint
for the stream instances operation typically these are written to a http.Request
*/
type StreamInstancesParams struct {

	/*EntityVersion
	  Entity version to start streaming from

	*/
	EntityVersion *string

	timeout    time.Duration
	Context    context.Context
	HTTPClient *http.Client
}

// WithTimeout adds the timeout to the stream instances params
func (o *StreamInstancesParams) WithTimeout(timeout time.Duration) *StreamInstancesParams {
	o.SetTimeout(timeout)
	return o
}

// SetTimeout adds the timeout to the stream instances params
func (o *StreamInstancesParams) SetTimeout(timeout time.Duration) {
	o.timeout = timeout
}

// WithContext adds the context to the stream instances params
func (o *StreamInstancesParams) WithContext(ctx context.Context) *StreamInstancesParams {
	o.SetContext(ctx)
	return o
}

// SetContext adds the context to the stream instances params
func (o *StreamInstancesParams) SetContext(ctx context.Context) {
	o.Context = ctx
}

// WithHTTPClient adds the HTTPClient to the stream instances params
func (o *StreamInstancesParams) WithHTTPClient(client *http.Client) *StreamInstancesParams {
	o.SetHTTPClient(client)
	return o
}

// SetHTTPClient adds the HTTPClient to the stream instances params
func (o *StreamInstancesParams) SetHTTPClient(client *http.Client) {
	o.HTTPClient = client
}

// WithEntityVersion adds the entityVersion to the stream instances params
func (o *StreamInstancesParams) WithEntityVersion(entityVersion *string) *StreamInstancesParams {
	o.SetEntityVersion(entityVersion)
	return o
}

// SetEntityVersion adds the entityVersion to the stream instances params
func (o *StreamInstancesParams) SetEntityVersion(entityVersion *string) {
	o.EntityVersion = entityVersion
}

// WriteToRequest writes these params to a swagger request
func (o *StreamInstancesParams) WriteToRequest(r runtime.ClientRequest, reg strfmt.Registry) error {

	r.SetTimeout(o.timeout)
	var res []error

	if o.EntityVersion != nil {

		// query param entityVersion
		var qrEntityVersion string
		if o.EntityVersion != nil {
			qrEntityVersion = *o.EntityVersion
		}
		qEntityVersion := qrEntityVersion
		if qEntityVersion != "" {
			if err := r.SetQueryParam("entityVersion", qEntityVersion); err != nil {
				return err
			}
		}

	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}
